/********		exp2.C

In this example file, we analyze the behavior of a nonlinear series RLC
circuit excited by a voltage source as shown below, where the C
is a nonlinear capacitor whose v-q characteristic is given by
v = -q + q^3, where q is the charge of the nonlinear capacitor.

	|------L---------R------------|      Vs(t) = f*cos(w*t)
	|      -----> i          +    |      L =  1.0
       Vs(t)                     v    NC      w, f, R parameters.
	|                        -    |
	|-----------------------------|

	q(t) : First state variable
	iL(t) = dq/dt: Second state variable
The set of differential equations for this circuit is known as
Duffing's equation, and known to be chaotic for some values of
the chosen parameters. (w=1.0 R=0.25, f=0.3)
********/


#include "head.h"
#include "memall.h"
#include "rk.h"
#include "gtools.h"
#include "four.h"


#define TNP 2048 /* Total # of points to be used for integration  */
#define NPS 1024 /* # of points to be stored for display	*/
#define NPV 2	/* # of variables to be plotted	*/
#define NV 2    /* # of state variables (order of system)	*/
#define NT 16	/* # of periods to be observed (t_end - t_begin = NT*T)  */
#define NPAR 3  /* # of parameters to be changed	*/

#define ULC 1 /* Upper left corner for display	*/
#define URC 2 /* Upper right corner for display	*/
#define LLC 3 /* Lower left corner for display	*/
#define LRC 4 /* Lower right corner for display	*/

char *parname="wfR";
/* Declare the system parameters globally */
float par[]={1.0,0.3,0.25};
float dpar[]={0.05,0.05,0.05};

/* User-supplied 'derivative' function	*/

void derivs(float x, float *yt, float *dyt)
{
		dyt[1] = yt[2];
dyt[2] = yt[1] - par[2]*yt[2] - yt[1]*yt[1]*yt[1] + par[1]*cos(par[0]*x);
}


/*	Main Program */
void main(void)
{
float x1,T,h,t;
int i,c,j,xgrid,ygrid,k,c2,ind;
int flag=1,flag2=1,flag4=1,flag5 = 0,flag6 = 1;
float *vs,*yout,*dyt,*yend;
float *time,**A;
float *fft,*xfft,*yfft;
float ti,tf,yi,yf,xstart=0.0;
unsigned int size;
FILE *ss;
char fname[12];
void far *buff;
		/* Allocate Memory for the arrays	*/
		fft = vector(TNP+2);
		A = matrix(NPV,NPS);
		time = vector(NPS);
		xfft = vector(NPS/2);
		yfft = vector(NPS/2);
		yend = vector(NV+1);
	vs=vector(NV+1);
	dyt = vector(NV+1);
	yout = vector(NV+1);
					for(k=0;k<=NV;k++)
				     yend[k] = 0.0;
		/*	Initial values for the system parameters	*/

	 yi = -2.0;  /* Default values for  Ymin, Ymax for the Y-axis,	*/
	 yf = 2.0;   /* Xgrid and Ygrid  */
	 xgrid = 5;
	 ygrid = 4;
			grafini(); /* Initialize graphics mode	*/

			clearviewport();
		   setcolor(12);
		 gprintfxy(100,240,"Running... Please Wait...");
		 settextstyle(SMALL_FONT,HORIZ_DIR,0);

				do{
					if (flag){
			T = PI2/par[0];
		h = T*(float) NT/TNP;

			if (flag6){
			  xstart = 0.0;
			     for(k=0;k<=NV;k++)
			  yend[k] = 0.0;
				}
	     for(k=0;k<=NV;k++)
		vs[k]=  yend[k];/*Initial conditions	*/
			x1 =  xstart; /* Initial time	*/


/* Allow NT*T time to pass to eliminate transients	*/
		if (flag6)
	for(i=1;i<=TNP;i++){
			derivs(x1,vs,dyt);
	rk4(vs,dyt,NV,x1,h,yout,derivs);
	     for(k=0;k<=NV;k++)
		vs[k]=  yout[k];
			x1 += h;
		}

			t = x1;
			       /*Solve and store for display	*/
	for(i=0;i<TNP;i++){
			derivs(x1,vs,dyt);
	rk4(vs,dyt,NV,x1,h,yout,derivs);
			if (!(i%2)){
			    j = i/2;
	     for(k=0;k<NPV;k++)
		A[k][j] =  yout[k+1];
			time[j] = t + i*h;
				}
		     for(k=0;k<=NV;k++)
		  vs[k]= yout[k];
			x1 += h;
		}
			/* Control to set Xmin,Xmax to defaults	*/
		if (flag4)
	 ti = t, tf = x1;

			if (flag5)
			   ti = t,tf = x1,flag5 = 0;

			}




/* Call Plotting utilities	*/

			 if (flag2){

			setviewport(0,0,639,479,1);
			clearviewport();


	plotxy(time,A[0],NPS-1,ti,tf,yi,yf,14,xgrid,ygrid,ULC);

	plotxy(time,A[1],NPS-1,ti,tf,yi,yf,12,xgrid,ygrid,URC);

	plotxy(A[0],A[1],NPS-1,yi,yf,yi,yf,2,ygrid,ygrid,LLC);
		setviewport(0,0,639,479,1);
			for(k=0;k<NPAR;k++)
		  gprintfxy(590,270+k*15,"%c=%.2f",parname[k],par[k]);
				}


/* Use key code mapping to be able to use Alt- and Ctrl- keys  */

		c = bioskey(0); /* Get user request	*/
/*Change system parameters	*/
			  c2 = (c & 0xFF);
			   if (isalpha(c2)){
			     ind = index(parname,c2);
				if (ind != -1){
  par[ind] = (isupper(c2))?par[ind]+dpar[ind]:par[ind]-dpar[ind];
	flag = flag2 = flag6 = 1;
			}
			else
			  flag2 = flag = 0;
				}
				else
			switch(c){



				/* Alt-i  */

	case 0x1700  : /*Increase the time interval by NT*T	*/
			flag6 = 0;
			  xstart = t = x1;
			    for(k=0;k<=NV;k++)
			  yend[k] = vs[k];
	setviewport(320,465,639,479,1);
	clearviewport();
	setcolor(12);
	  gprintf("Running.. Wait..");
	for(i=0;i<TNP;i++){
			derivs(x1,vs,dyt);
	rk4(vs,dyt,NV,x1,h,yout,derivs);
			if (!(i%2)){
				j = i/2;
			      for(k=0;k<NPV;k++)
				A[k][j] = yout[k+1];
		time[j] = t + i*h;
				}
				   for(k=0;k<=NV;k++)
				  vs[k] = yout[k];
			x1 += h;
		}
		ti = t;
		tf = x1;

		  flag = 0;
		  flag2 = 1;
		  flag4 = 1;
		  flag5 = 1;
		  break;

				/* Alt-x */
	case 0x2d00 : /*Change Xmin, Xmax	*/

		setviewport(320,465,639,479,1);
		clearviewport();
  gprintf("Enter Xmin and Xmax:");
		      gscanf("%f %f",&ti,&tf);
		 flag = 0;
		 flag2 = 1;
		 flag4 = 0;
		   if (ti == 999.0 && tf == 999.0){
			ti = t;
			tf = x1;
			   flag4 = 1;
			     }



			break;

				/* Alt-y */
	case 0x1500 :	/* Change Ymin,Ymax	*/

			setviewport(320,465,639,479,1);
			clearviewport();
		  gprintf("Enter Ymin and Ymax:");
		      gscanf("%f %f",&yi,&yf);
			flag = 0;
			flag2 = 1;
			break;

			/* Alt-g  */
			/* To change Xgrid,Ygrid  */
	case 0x2200 :
			setviewport(320,465,639,479,1);
		  gprintf("Enter Xgrid and Ygrid:");
		      gscanf("%d %d",&xgrid,&ygrid);
			flag = 0;
			flag2 = 1;
			break;

	case 0x1f00 :	/* Alt-s  to save screen to a file  */

			setviewport(320,465,639,479,1);
			clearviewport();
		  gprintf("Screen Dump File Name(Ext .IMG):");
		      gscanf("%s",fname);
		      if ((ss = gopen(fname,"wb",".img")) != NULL){
				clearviewport();
		       setviewport(0,0,639,479,1);
				sscreen(ss);
			setviewport(320,465,639,479,1);
			clearviewport();
	gprintfxy(5,0,"Screen image saved to '%s'",fname);
			      fclose(ss);
				}
				else{
				  clearviewport();
	gprintfxy(5,0,"Invalid filename -- Screen not saved");
				}
			flag = 0;
			flag2 = 0;
			break;

	case 0x1300 : /* Alt-r, to restore a previously saved screen */
			setviewport(320,465,639,479,1);
			clearviewport();
		  gprintf("File Name to be Restored(Ext .IMG):");
		      gscanf("%s",fname);
		      if ((ss = gopen(fname,"rb",".img")) != NULL){
		       setviewport(0,0,639,479,1);
				clearviewport();
				rscreen(ss);
			setviewport(320,465,639,479,1);
			clearviewport();
	gprintfxy(5,0,"Screen image restored from '%s'",fname);
			      fclose(ss);
				}
				else{
				  clearviewport();
	gprintfxy(5,0,"Invalid filename -- Screen not restored");
					}

			flag = 0;
			flag2 = 0;
			break;

	case 0x1400 : /* Alt-t,  Do FFT  */
			setviewport(320,465,639,479,1);
			clearviewport();
			setcolor(12);
			gprintfxy(10,0,"FFT... Wait...");
		  for(j=0;j<NPS;j++){
		    fft[2*j+1] = A[0][j];
		    fft[2*j+2] = 0.0;
				}
		       fourout(fft,xfft,yfft,NPS,2*h);
			  clearviewport();
	 plotxy(xfft,yfft,NPS/2-1,0,5,-50,0,14,5,5,LRC);

		flag = 0;
		flag2 = 0;
			break;


	case 0x2300 : /* Help	*/
			setviewport(350,250,550,450,1);
			size = imagesize(0,0,200,200);
			buff = farmalloc(size*sizeof(char));
			getimage(0,0,200,200,buff);
			clearviewport();
				setcolor(12);
			    for(i=0;i<9;i++)
		gprintfxy(0,i*15,"%s",help[i]);
		 gprintfxy(10,160,"%s",help[9]);
			getch();
			clearviewport();
			putimage(0,0,buff,COPY_PUT);
			   farfree(buff);
			   setviewport(320,465,639,479,1);
			     flag = 0;
			     flag2 = 0;
			       break;


	default:        flag = 0;
			flag2 = 0;
			break;
		}
			if (flag){
		setviewport(320,465,639,479,1);
		clearviewport();
		   setcolor(12);
		 gprintf("Running.. Wait...");

					   }
					   /* ESC to exit */
			}while( c != 0x011b); /* Job Done */

		closegraph();   /* End Graphics mode	*/

		/* Free allocated memory	*/
		free_vector(fft);
	free_matrix(A,NPV);
	free_vector(time);
	free_vector(xfft);
	free_vector(yfft);
	free_vector(vs);
	free_vector(dyt);
	free_vector(yout);
	free_vector(yend);
		return;	/* End Program	*/
}





































